#include <tuple>
#include <iostream>
#include <opencv2/opencv.hpp>
#include <opencv2/dnn.hpp>
#include <iterator>
using namespace cv;
using namespace cv::dnn;
using namespace std;

/**
 * @brief Get the Face Box object 人脸定位
 *
 * @param net 人脸检测网络
 * @param frame 检测图像
 * @param conf_threshold 阈值
 * @return tuple<Mat, vector<vector<int>>> 元组容器，可返回多个值
 */
tuple<Mat, vector<vector<int>>> getFaceBox(Net net, Mat &frame, double conf_threshold)
{
    //图像复制
    Mat frameOpenCVDNN = frame.clone();
    int frameHeight = frameOpenCVDNN.rows;
    int frameWidth = frameOpenCVDNN.cols;
    //缩放尺寸
    double inScaleFactor = 1.0;
    //检测图大小
    Size size = Size(300, 300);
    // std::vector<int> meanVal = {104, 117, 123};
    Scalar meanVal = Scalar(104, 117, 123);

    cv::Mat inputBlob;
    inputBlob = cv::dnn::blobFromImage(frameOpenCVDNN, inScaleFactor, size, meanVal, true, false);
    net.setInput(inputBlob, "data");
    //四维矩阵输出
    cv::Mat detection = net.forward("detection_out");
    //提取结果信息
    cv::Mat detectionMat(detection.size[2], detection.size[3], CV_32F, detection.ptr<float>());

    vector<vector<int>> bboxes;

    for (int i = 0; i < detectionMat.rows; i++)
    {
        //预测概率
        float confidence = detectionMat.at<float>(i, 2);

        if (confidence > conf_threshold)
        {
            //左上角点，坐标被归一化
            int x1 = static_cast<int>(detectionMat.at<float>(i, 3) * frameWidth);
            int y1 = static_cast<int>(detectionMat.at<float>(i, 4) * frameHeight);
            //右下角角点，坐标被归一化
            int x2 = static_cast<int>(detectionMat.at<float>(i, 5) * frameWidth);
            int y2 = static_cast<int>(detectionMat.at<float>(i, 6) * frameHeight);
            vector<int> box = { x1, y1, x2, y2 };
            //人脸坐标
            bboxes.push_back(box);
            //图像框选
            cv::rectangle(frameOpenCVDNN, cv::Point(x1, y1), cv::Point(x2, y2), cv::Scalar(0, 255, 0), 2, 4);
        }
    }

    return make_tuple(frameOpenCVDNN, bboxes);
}

#include <fstream>
#include "Poco/StringTokenizer.h"
#include "Poco/Stopwatch.h"
Scalar MODEL_MEAN_VALUES = Scalar(104, 117, 123);

int main(void) {
    string modelBaseDir = "/Users/edz/Downloads/";
    string genderProto = modelBaseDir+"genderModel/gender.prototxt";
    string genderModel = modelBaseDir+"genderModel/gender.caffemodel";
    //性别标签
    vector<string> genderList = {"Male", "Female"};
    //导入网络
    Net genderNet = cv::dnn::readNet(genderProto, genderModel);
    ifstream ifs("/Users/edz/Downloads/wiki_crop/gender.txt");
    if(!ifs.is_open()){
        cout << "文件未打开成功" << endl;
        return 0;
    }
    Poco::Stopwatch stopwatch;
    stopwatch.start();
    vector<int> pV;
    vector<string> nV;
    string line;
    int count = 0;
    while(getline(ifs,line)){
        Poco::StringTokenizer stringTokenizer(line," ");
        string fileName = stringTokenizer[0];
        if(fileName!="61/22156261_1992-10-16_2011.jpg"){
            continue;
        }
        int gender = stoi(stringTokenizer[1]);
        Mat face;
        face = imread("/Users/edz/Downloads/wiki_crop/"+fileName);
        //性别检测
        Mat blob;
        blob = blobFromImage(face, 1, Size(224, 224), MODEL_MEAN_VALUES, true);
        genderNet.setInput(blob);
        // string gender_preds; 获取前向传播softmax结果
        vector<float> genderPreds = genderNet.forward();
        // find max element index max_element用于找寻最大值
        // distance function does the argmax() work in C++ distance返回最大值和第一个值下标的距离
        int max_index_gender = std::distance(genderPreds.begin(),
                                             max_element(genderPreds.begin(), genderPreds.end()));
        //获得检测结果
        pV.push_back(max_index_gender==gender);
        if(max_index_gender!=gender){
            nV.push_back("/Users/edz/Downloads/wiki_crop/"+fileName);
        }
//        string gender = genderList[max_index_gender];
//        cout << "Gender: " << gender << endl;
        ++count;
        cout << int(stopwatch.elapsed()/1000) << endl;
        imshow("bug",face);
        waitKey(0);
        if(count==1){
            break;
        }
    }
    int nC = 0;
    for(auto it:pV){
        if(it==0){
            ++nC;

        }
    }
    for(auto it:nV){
        cout << it << endl;
    }
    cout << "正确率:" << (pV.size()-nC)*1.0/pV.size() << endl;
    cout << "总耗时:" << int(stopwatch.elapsed()/1000) << endl;
    ifs.close();


    return 0;
}
int main22(void) {
    string modelBaseDir = "/Users/edz/Downloads/";
    string genderProto = modelBaseDir+"genderModel/gender.prototxt";
    string genderModel = modelBaseDir+"genderModel/gender.caffemodel";
    //性别标签
    vector<string> genderList = {"Male", "Female"};
    //导入网络
    Net genderNet = cv::dnn::readNet(genderProto, genderModel);
    ifstream ifs("/Users/edz/Downloads/wiki_crop/gender.txt");
    if(!ifs.is_open()){
        cout << "文件未打开成功" << endl;
        return 0;
    }
    Poco::Stopwatch stopwatch;
    stopwatch.start();
    vector<int> pV;
    vector<string> nV;
    string line;
    int totalCount = 1000;
    int count = 0;
    while(getline(ifs,line)){
        Poco::StringTokenizer stringTokenizer(line," ");
        string fileName = stringTokenizer[0];
        int gender = stoi(stringTokenizer[1]);
        Mat face;
        face = imread("/Users/edz/Downloads/wiki_crop/"+fileName);
        cout << "总耗时1:" << int(stopwatch.elapsed()/1000) << "ms" << endl;
        //性别检测
        Mat blob;
        blob = blobFromImage(face, 1, Size(224, 224), MODEL_MEAN_VALUES, true);
        cout << "总耗时2:" << int(stopwatch.elapsed()/1000) << "ms" << endl;
        genderNet.setInput(blob);
        cout << "总耗时3:" << int(stopwatch.elapsed()/1000) << "ms" << endl;
        // string gender_preds; 获取前向传播softmax结果
        vector<float> genderPreds = genderNet.forward();
        cout << "总耗时4:" << int(stopwatch.elapsed()/1000) << "ms" << endl;
        // find max element index max_element用于找寻最大值
        // distance function does the argmax() work in C++ distance返回最大值和第一个值下标的距离
        int max_index_gender = std::distance(genderPreds.begin(),
                                             max_element(genderPreds.begin(), genderPreds.end()));
        cout << "总耗时5:" << int(stopwatch.elapsed()/1000) << "ms" << endl;
        //获得检测结果
        pV.push_back(max_index_gender==gender);
        cout << "总耗时6:" << int(stopwatch.elapsed()/1000) << "ms" << endl;
        if(max_index_gender!=gender){
            nV.push_back("/Users/edz/Downloads/wiki_crop/"+fileName);
        }
//        string gender = genderList[max_index_gender];
//        cout << "Gender: " << gender << endl;
        ++count;
        cout << "已完成:" << count*1.0*100/totalCount << "%" << endl;
        if(count==totalCount){
            break;
        }
    }
    int nC = 0;
    for(auto it:pV){
        if(it==0){
            ++nC;

        }
    }
    for(auto it:nV){
        cout << it << endl;
    }

    cout << "正样本数:" << count-nC << endl;
    cout << "负样本数:" << nC << endl;
    cout << "总样本数:" << count << endl;
    cout << "正确率:" << (pV.size()-nC)*1.0*100/pV.size() << "%" << endl;
    cout << "总耗时:" << int(stopwatch.elapsed()/1000) << "ms" << endl;
    cout << "单次平均耗时:" << int(stopwatch.elapsed()/1000/count) << "ms" << endl;
    ifs.close();


    return 0;
}
int main2(void) {
    //人脸模型
    string modelBaseDir = "/Users/edz/Downloads/";
    string faceProto = modelBaseDir+"2opencv_dlib_face_detection/model/opencv_face_detector.pbtxt";
    string faceModel = modelBaseDir+"2opencv_dlib_face_detection/model/opencv_face_detector_uint8.pb";

    //年龄模型
//    string ageProto = modelBaseDir+"1age_gender/age_deploy.prototxt";
//    string ageModel = modelBaseDir+"1age_gender/age_net.caffemodel";

    //性别模型
//    string genderProto = modelBaseDir+"1age_gender/gender_deploy.prototxt";
//    string genderModel = modelBaseDir+"1age_gender/gender_net.caffemodel";
    string genderProto = modelBaseDir+"genderModel/gender.prototxt";
    string genderModel = modelBaseDir+"genderModel/gender.caffemodel";

    //均值
    Scalar MODEL_MEAN_VALUES = Scalar(104, 117, 123);

    //年龄段标签
    vector<string> ageList = {"(0-2)", "(4-6)", "(8-12)", "(15-20)", "(25-32)",
                              "(38-43)", "(48-53)", "(60-100)"};

    //性别标签
    vector<string> genderList = {"Female", "Male"};

    //导入网络
//    Net ageNet = cv::dnn::readNet(ageProto, ageModel);
    Net genderNet = cv::dnn::readNet(genderProto, genderModel);
    Net faceNet = cv::dnn::readNetFromTensorflow(faceModel, faceProto);

    //打开摄像头
    VideoCapture cap;
    cap.open(0);
    if (cap.isOpened()) {
        cout << "camera is opened!" << endl;
    } else {
        return 0;
    }

    int padding = 20;
    while (waitKey(1) < 0) {
        // read frame 读图
        Mat frame;
        cap.read(frame);
        if (frame.empty()) {
            waitKey();
            break;
        }
        cout << frame.type() << endl;
//        frame = imread("./images/couple1.jpg");
        //人脸坐标
        vector<vector<int>> bboxes;
        //人脸检测结果图
        Mat frameFace;
        //人脸定位
        //tie()函数解包frameFace和bboxes
        tie(frameFace, bboxes) = getFaceBox(faceNet, frame, 0.7);
        //人脸判断
        if (bboxes.size() == 0) {
            cout << "No face detected, checking next frame." << endl;
            continue;
        }
        cout << "face detected, checking next frame." << endl;
        //逐个提取人脸检测
        for (auto it = begin(bboxes); it != end(bboxes); ++it) {
            //框选人脸
            Rect rec(it->at(0) - padding, it->at(1) - padding, it->at(2) - it->at(0) + 2 * padding,
                     it->at(3) - it->at(1) + 2 * padding);
            //避免人脸框选超过图像边缘
            rec.width = ((rec.x + rec.width) > frame.cols) ? (frame.cols - rec.x - 1) : rec.width;
            rec.height = ((rec.y + rec.height) > frame.rows) ? (frame.rows - rec.y - 1) : rec.height;

            // take the ROI of box on the frame,原图中提取人脸
            Mat face = frame(rec);

            //性别检测
            Mat blob;
            blob = blobFromImage(face, 1, Size(224, 224), MODEL_MEAN_VALUES, true);
            cout << "genderNet start." << endl;
            cout << blob.size << endl;
            cout << blob.rows << endl;
            cout << blob.cols << endl;
            cout << blob.dims << endl;
            genderNet.setInput(blob);
            // string gender_preds; 获取前向传播softmax结果
            vector<float> genderPreds = genderNet.forward();
            cout << "genderNet end." << endl;
            // find max element index max_element用于找寻最大值
            // distance function does the argmax() work in C++ distance返回最大值和第一个值下标的距离
            int max_index_gender = std::distance(genderPreds.begin(),
                                                 max_element(genderPreds.begin(), genderPreds.end()));
            //获得检测结果
            string gender = genderList[max_index_gender];
            cout << "Gender: " << gender << endl;

            //年龄识别
//            blob = blobFromImage(face, 1, Size(227, 227), MODEL_MEAN_VALUES, false);
//            ageNet.setInput(blob);
//            vector<float> agePreds = ageNet.forward();
//            // finding maximum indicd in the age_preds vector 找到年龄预测最大下表
//            int max_indice_age = std::distance(agePreds.begin(), max_element(agePreds.begin(), agePreds.end()));
//            string age = ageList[max_indice_age];
//            cout << "Age: " << age << endl;

            // label 输出标签
//            string label = gender + ", " + age;
            string label = gender;
            //在人脸定位图上显示结果
            cv::putText(frameFace, label, Point(it->at(0), it->at(1) - 15), cv::FONT_HERSHEY_SIMPLEX, 0.9,
                        Scalar(0, 255, 255), 2, cv::LINE_AA);
        }
        //保存结果
        imshow("Frame", frameFace);
//        imwrite("out.jpg", frameFace);
    }
}



int main3(void) {
    //人脸模型
    string modelBaseDir = "/Users/edz/Downloads/";
    string faceProto = modelBaseDir+"2opencv_dlib_face_detection/model/opencv_face_detector.pbtxt";
    string faceModel = modelBaseDir+"2opencv_dlib_face_detection/model/opencv_face_detector_uint8.pb";

    //年龄模型
    string ageProto = modelBaseDir+"1age_gender/age_deploy.prototxt";
    string ageModel = modelBaseDir+"1age_gender/age_net.caffemodel";

    //性别模型
//    string genderProto = modelBaseDir+"1age_gender/gender_deploy.prototxt";
//    string genderModel = modelBaseDir+"1age_gender/gender_net.caffemodel";
    string genderProto = modelBaseDir+"genderModel/gender.prototxt";
    string genderModel = modelBaseDir+"genderModel/gender.caffemodel";

    //均值
    Scalar MODEL_MEAN_VALUES = Scalar(78.4263377603, 87.7689143744, 114.895847746);

    //年龄段标签
    vector<string> ageList = {"(0-2)", "(4-6)", "(8-12)", "(15-20)", "(25-32)",
                              "(38-43)", "(48-53)", "(60-100)"};

    //性别标签
    vector<string> genderList = {"Male", "Female"};

    //导入网络
    Net ageNet = cv::dnn::readNet(ageProto, ageModel);
    Net genderNet = cv::dnn::readNet(genderProto, genderModel);
    Net faceNet = cv::dnn::readNetFromTensorflow(faceModel, faceProto);

    //打开摄像头
    VideoCapture cap;
    cap.open(0);
    if (cap.isOpened()) {
        cout << "camera is opened!" << endl;
    } else {
        return 0;
    }

    int padding = 20;
    while (waitKey(1) < 0) {
        // read frame 读图
        Mat frame;
        cap.read(frame);
        if (frame.empty()) {
            waitKey();
            break;
        }
//        frame = imread("./images/couple1.jpg");
        //人脸坐标
        vector<vector<int>> bboxes;
        //人脸检测结果图
        Mat frameFace;
        //人脸定位
        //tie()函数解包frameFace和bboxes
        tie(frameFace, bboxes) = getFaceBox(faceNet, frame, 0.7);
        //人脸判断
        if (bboxes.size() == 0) {
            cout << "No face detected, checking next frame." << endl;
            continue;
        }
        //逐个提取人脸检测
        for (auto it = begin(bboxes); it != end(bboxes); ++it) {
            //框选人脸
            Rect rec(it->at(0) - padding, it->at(1) - padding, it->at(2) - it->at(0) + 2 * padding,
                     it->at(3) - it->at(1) + 2 * padding);
            //避免人脸框选超过图像边缘
            rec.width = ((rec.x + rec.width) > frame.cols) ? (frame.cols - rec.x - 1) : rec.width;
            rec.height = ((rec.y + rec.height) > frame.rows) ? (frame.rows - rec.y - 1) : rec.height;

            // take the ROI of box on the frame,原图中提取人脸
            Mat face = frame(rec);

            //性别检测
            Mat blob;
            blob = blobFromImage(face, 1, Size(227, 227), MODEL_MEAN_VALUES, false);
            genderNet.setInput(blob);
            // string gender_preds; 获取前向传播softmax结果
            vector<float> genderPreds = genderNet.forward();
            // find max element index max_element用于找寻最大值
            // distance function does the argmax() work in C++ distance返回最大值和第一个值下标的距离
            int max_index_gender = std::distance(genderPreds.begin(),
                                                 max_element(genderPreds.begin(), genderPreds.end()));
            //获得检测结果
            string gender = genderList[max_index_gender];
            cout << "Gender: " << gender << endl;

            //年龄识别
            ageNet.setInput(blob);
            vector<float> agePreds = ageNet.forward();
            // finding maximum indicd in the age_preds vector 找到年龄预测最大下表
            int max_indice_age = std::distance(agePreds.begin(), max_element(agePreds.begin(), agePreds.end()));
            string age = ageList[max_indice_age];
            cout << "Age: " << age << endl;

            // label 输出标签
            string label = gender + ", " + age;
            //在人脸定位图上显示结果
            cv::putText(frameFace, label, Point(it->at(0), it->at(1) - 15), cv::FONT_HERSHEY_SIMPLEX, 0.9,
                        Scalar(0, 255, 255), 2, cv::LINE_AA);
        }
        //保存结果
        imshow("Frame", frameFace);
        imwrite("out.jpg", frameFace);
    }
}
